
% REPRODUCING EXPERIMENT 1

clear; % we need to clear the variables in workspace before learning. 

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
epsilon = .1;  % allowed correlation between selected Gabors
T = 0.;        % threshold for responses
Totalsketch = 48;  % allowed number of selected Gabors   
Upperbound = 16.; % saturation value
sub = 2;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
%%% Note: Lrange*sub is the pixels allowed to shift along normal
%%%       Lrange*sub should be the same in learning and testing

LIK = 1.;  % use log-likelihood score. If LIK=0, then use active correlation
% LIK = 0.;  

% TRAINING EXAMPLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Iname = {'car5.jpg' 'car6.jpg' 'car7.jpg' 'car8.jpg' 'car9.jpg' 'car10.jpg' 'car11.jpg' 'car12.jpg' 'car13.jpg' 'car14.jpg' 'car15.jpg' 'car16.jpg' 'car17.jpg' 'car18.jpg' 'car19.jpg' 'car20.jpg' 'car21.jpg' 'car22.jpg' 'car23.jpg' 'car24.jpg' 'car25.jpg' 'car26.jpg' 'car27.jpg' 'car28.jpg' 'car29.jpg' 'car30.jpg' 'car31.jpg' 'car32.jpg' 'car33.jpg' 'car34.jpg' 'car35.jpg' 'car36.jpg' 'car37.jpg' 'car38.jpg' 'car39.jpg' 'car40.jpg' 'car41.jpg'};
nimage = size(Iname, 2);
Isize = zeros(nimage, 2); 
for (i = 1 : nimage)
    I{i} = imresize(double(rgb2gray(imread(Iname{i}))), 1); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for (i = 1 : nimage)
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the training images to be the same



% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
C = corr(allfilter, epsilon);    % generate the inhibition maps
allfiltered = applyfilter(I, allfilter, T);  % filter training images



% VARIABLES FOR MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % average response of selected Gabor
Mo = zeros(nimage, Totalsketch); % shifting in orientation
Ml = zeros(nimage, Totalsketch); % shifting in location
sym = zeros(size(I{1}));  % symbolic plot of selected Gabor
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for (i = 1 : nimage)
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, 1);  % total scores for testing images
gain = zeros(Totalsketch, 1);  % coding gain for selected Gabor



% MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mex Clearn.c;   % compile C code
Clearn(nimage, norient, allfiltered, C, h, sx, sy, T, Mi, Mx, My, Mm, allsymbol(1, :), sym, Asym, Lrange, Orange, sub, Totalsketch, Upperbound, tss, Mo, Ml, gain, LIK);



% RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
K = nimage;
[tsssort, ind] = sort(tss); 
figure; % display top K examples
col = 10; row = floor(K/col*2)+1; row = col;
subplot(row, col, 1, 'v6'); imshow(-(sym), []); 
for (i = 1 : K)
    subplot(row, col,  1+i*2, 'v6'); 
    imshow(I{ind(nimage-i+1)}, []);
    subplot(row, col,  2+i*2, 'v6');
    imshow(-(Asym{ind(nimage-i+1)}), []); 
end
figure; 
plot(gain); % plot coding gain curve
