/**************************************************
 Mex C Code for reproducing experiment 1
***************************************************/

# include "mex.h"        /* the algorithm is connect to matlab */
# include "math.h"
# define PI 3.1415926
# define ABS(x) ((x)>0? (x):(-(x)))
# define MAX(x, y) ((x)>(y)? (x):(y))
# define MIN(x, y) ((x)<(y)? (x):(y))

/* getting the index of matlab image, (x,y) location, (sx, sy) sizes */
int px(int x, int y, int bx, int by)    
{            
   return (x + (y-1)*bx - 1); 
 }


 /* variables */
 int n;                      /* number of images */
 int N;                      /* number of orientations */
 double **fI;                /* filtered images */
 double **C;                 /* inhibition coefficients */
 int h;                      /* halfsizes of filters */
 int sx, sy;                 /* sizes of image */ 
 double T;                   /* threshold of selected bases */
 double *Mi, *Mx, *My, *Mm;  /* storing selected bases */
 double **allsymbol;         /* symbol of filters */
 double *sym, **Asym;        /* symbol of selected Gabors */
 int L, ore;                 /* allowed ranges of shifting in location and orientation */
 int sub;                    /* subsampling */
 int Totalsketch;            /* total number of Gabors */
 double Upperbound;          /* If response is greater than Upperbound, then set response = Upperbound */
 double *tss;                /* score for each example */
 double *Mo, *Ml;            /* shifts in orientation and location for each example */
 double *gain;               /* coding gain for each selected base per example */
 int LIK;   /* If LIK = 1, we use log-likelihood score. If LIK = 0, we use active correlation */

/* for Gabor(x, y, orientation = ind), find local maximum in image i */
double shiftmax(int i, int ind, int x, int y, int *rm) 
{
   double theta, m;
   int x1, y1, l, ml, mx, my, de, d, d1, here, mo, md; 
 
   theta = PI*ind/N;   /* orientation */
   m = 0.; 
   for (l=-L; l<=L; l++)   
     {
        x1 = floor(l*sub*cos(theta)+x+.5); 
        y1 = floor(l*sub*sin(theta)+y+.5); /* shifting in normal direction */
        if ((x1>=1)&&(x1<=sx)&&(y1>=1)&&(y1<=sy))
        {
        here = px(x1, y1, sx, sy);
        for (de=-ore; de<=ore; de++)
            {
              d = de+ind;
              d1 = d;
              if (d<0)
                 d1 = d+N;
              if (d>=N)
                  d1 = d-N;   /* shifting in orientation */
              if (fI[d1*n+i][here]>m)
                {
                   m = fI[d1*n+i][here];   /* local maximization */
                   ml = l; mx = x1; my = y1; mo = d1; md = de; 
                }
             }
        }
     }
   rm[0] = ml; rm[1] = mx; rm[2] = my; rm[3] = mo; rm[4] = md; 
   return(MIN(Upperbound, m)); 
}


/* the Gabor(mx, my, orientation = mi) inhibits overlapping Gabors for image i */
void inhibit(int i, int mi, int mx, int my) 
{
   int x, y, ind; 
   double *f, *fc; 
   
   for (ind=0; ind<N; ind++)   
     {
       f = fI[ind*n+i];   
       fc = C[mi+ind*N];   /* inhibition coefficients, zero = inhibit */
       for (x=MAX(1, mx-2*h); x<=MIN(sx, mx+2*h); x++)
         for (y=MAX(1, my-2*h); y<=MIN(sy, my+2*h); y++)
         {
          f[px(x, y, sx, sy)] *= fc[px(x-mx+2*h+1, y-my+2*h+1, 4*h+1, 4*h+1)];
         }
      }
}


/* draw the symbol of Gabor(x0, y0, orientation = ind) with intensity w */
void draw(double *sym, int x0, int y0, int ind, double w)
{
  int x, y; 
  double a; 
          
  for (x=x0-h; x<=x0+h; x++)
     for (y=y0-h; y<=y0+h; y++)
       {
         a = allsymbol[ind][px(x-x0+h+1, y-y0+h+1, 2*h+1, 2*h+1)]*w; 
         if (sym[px(x, y, sx, sy)]<a)
             sym[px(x, y, sx, sy)] = a;
       }
}


/* the shared sketch algorithm */
void Cmp()
{
   int i, ind, x, y, mi, mx, my, t, rm[5], here; 
   double m, s, mm; 

   t = 0; 
   do  
   {
     m = 0.; 
     for (ind=0; ind<N; ind++)
     { 
        for (x=floor((h+1)/sub)+1+L; x<=floor((sx-h)/sub)-1-L; x++)
           for (y=floor((h+1)/sub)+1+L; y<=floor((sy-h)/sub)-1-L; y++)
             {
               s = 0.; 
               for (i=0; i<n; i++)
                 {
                   mm = shiftmax(i, ind, x*sub, y*sub, rm);   /* local maximum of active Gabor */   
                   if (LIK)
                   {
                   s += mm; 
                   }
                   else
                   {
                    s += sqrt(mm); 
                   }
                  }   
                if (m<s)
                  {
                    m = s; 
                    mi = ind; mx = x*sub; my = y*sub; 
                   }
                }
         }
     Mi[t] = mi; Mx[t] = mx; My[t] = my; 
     Mm[t] = m/n;   

     draw(sym, mx, my, mi, Mm[t]);       
           
     for (i=0; i<n; i++)
        {            
          mm = shiftmax(i, mi, mx, my, rm);    
         
          if (LIK)
          {
          tss[i] += (1. - 1./Mm[t])*mm - log(Mm[t]); 
          }
          else
          {
          tss[i] += sqrt(mm)*Mm[t]; 
          }
           
          if (mm>0.)
            {  
               draw(Asym[i], rm[1], rm[2], rm[3], sqrt(mm)); 
               here = px(i+1, t+1, n, Totalsketch);
               Mo[here] = rm[4];
               Ml[here] = rm[0];
               inhibit(i, rm[3], rm[1], rm[2]); 
             }
        }
     if (LIK)
     {
      gain[t] = Mm[t]-1.-log(Mm[t])-log(2.0*L+1.0) - log(2.0*ore+1.0); 
     }
     else
     {
      gain[t] = Mm[t]; 
     }
   
      printf("%d:  (orient = %d   x = %d   y = %d) --> %f\n", t, mi, mx, my, gain[t]);
      t++; 
   }
  while (t<Totalsketch);   /* can also stop if gain[t]<0 */

}


/* mex function is used to pass on the pointers and scalars from matlab, 
   so that heavy computation can be done by C, which puts the results into 
   some of the pointers. After that, matlab can then use these results. 
   
   So matlab is very much like a managing platform for organizing the 
   experiments, and mex C is like a work enginee for fast computation. */

void mexFunction(int nlhs, mxArray *plhs[], 
                 int nrhs, const mxArray *prhs[])                
{
 int ind, i, j, c; 
 mxArray *f;  
 
 c = 0; /* counter for input pointers and scalars */
 n = floor(mxGetScalar(prhs[c++])+.5);  /* number of images */
 N = floor(mxGetScalar(prhs[c++])+.5);  /* number of orientations */
 fI = mxCalloc(n*N, sizeof(double*));   /* fitered images */
 for (i=0; i<n; i++)
   {
     for (ind=0; ind<N; ind++)
      {  
       f = mxGetCell(prhs[c], ind*n+i); 
       fI[ind*n+i] = mxGetPr(f);    /* get pointers to filtered images */       
      }
    }
 c++; 
 
 C = mxCalloc(N*N, sizeof(double*));    /* C: correlation/inhibition between filters */
 for (ind=0; ind<N; ind++)
     {  
       for (j=0; j<N; j++)
        {
         f = mxGetCell(prhs[c], j*N+ind); 
         C[j*N+ind] = mxGetPr(f);         /* get correlation/inhibition */
        }   
     }
 c++; 
 
 h = floor(mxGetScalar(prhs[c++])+.5);    /* half size of filters */
 
 sx = floor(mxGetScalar(prhs[c++])+.5);
 sy = floor(mxGetScalar(prhs[c++])+.5);   /* size of images */
 T = mxGetScalar(prhs[c++]);              /* threshold */  
     
 Mi = mxGetPr(prhs[c++]);   /* orientation of selected Gabor */               
 Mx = mxGetPr(prhs[c++]);         
 My = mxGetPr(prhs[c++]);   /* (x, y) position of selected Gabor */
 Mm = mxGetPr(prhs[c++]);   /* average response of selected Gabor */    

 allsymbol = mxCalloc(N, sizeof(double*));    
 for (ind=0; ind<N; ind++)
     {  
       f = mxGetCell(prhs[c], ind); 
       allsymbol[ind] = mxGetPr(f);  /* symbols of filters */         
     }
 c++; 
 
 sym = mxGetPr(prhs[c++]);   /* symbols of selected Gabors for all the images */          
 Asym = mxCalloc(n, sizeof(double*));   /* symbols of active Gabors for an individual image */
 for (i=0; i<n; i++)
    {
        f = mxGetCell(prhs[c], i);
        Asym[i] = mxGetPr(f);  
     }
 c++; 
 
 L = floor(mxGetScalar(prhs[c++])+.5);     /* range of shifting along normal direction of Gabor */
 ore = floor(mxGetScalar(prhs[c++])+.5);   /* range of shifting in orientation */
 sub = floor(mxGetScalar(prhs[c++])+.5);   /* sub-sampling of Gabor filters to improve speed */

 Totalsketch = floor(mxGetScalar(prhs[c++])+.5);  /* total number of bases to be selected */
 Upperbound = mxGetScalar(prhs[c++]);       /* saturation level */
 tss = mxGetPr(prhs[c++]);                  /* log-likelihood scores for examples */

 Mo = mxGetPr(prhs[c++]);                   /* shift in orientation */
 Ml = mxGetPr(prhs[c++]);                   /* shift in location */
 gain = mxGetPr(prhs[c++]);                 /* coding gain for selected base */
 
 LIK = floor(mxGetScalar(prhs[c++])+.5);    /* log-likelihood or active correlation? */
 
 Cmp();
}

     



 

                    